package require ncgi

namespace eval b_write_calc_file {
    variable doc
}

proc b_write_calc_file::writeCalcFile { _doc file } {
    variable doc    
    set doc $_doc
    set root [$doc documentElement]
    write_calc_data init $file
    write_ouptut_file $root
    write_calc_data end
}

# proc b_write_calc_file::get_value { name } {
#     variable doc
#     
#     set node [$doc selectNodes [format_xpath {//value[@n=%s]} $name]]
#     return [get_domnode_attribute $node v]
# }

################################################################################
#      CODE TO PRINT THE OUTPUT FILE (OLD .BAS)
################################################################################

proc b_write_calc_file::write_ouptut_file { root } {  
    
    write_calc_data puts -nonewline "=================================================================="
    write_calc_data puts -nonewline "\n                        General Data File"    
    write_calc_data puts -nonewline "\n=================================================================="
    
    write_calc_data puts -nonewline "\n\n%%%%%%%%%%%%%%%%%% Problem Size  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%"
    write_calc_data puts -nonewline "\nNumber of Elements & Nodes:"
    write_calc_data puts -nonewline "\n[GiD_Info Mesh NumElements] [GiD_Info Mesh NumNodes]"
    
    write_calc_data puts -nonewline "\n\n%%%%%%%%%%%%%%%%%%% Mesh Database  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%"
    
    #################### COORDINATES ############################ 
    write_calc_data puts -nonewline "\n\nCoordinates:"
    write_calc_data puts -nonewline "\n  Node        X                Y"
     
    # Geometry factor (here the geometry unit selected by the user is converted to 'cm')  
    set units_length [gid_groups_conds::give_active_unit L]
    
    set xp [format_xpath {/*/units/unit_magnitude[@n="L"]/unit[@n=%s]} "m"]
    set node [gid_groups_conds::give_node_xpath $xp]
    set L_geom_fac [get_domnode_attribute $node factor]
    
    # We call C functions from the COMPASSLIB to print everything related with nodes or elements        
    write_calc_data coordinates -factor $L_geom_fac "\n%5d %14.5E %14.5E%.0s"
    
    #################### CONNECTIVITIES ############################
    
    write_calc_data puts -nonewline "\n\n................................................................."    
    write_calc_data puts -nonewline "\n\nConnectivities:"
    write_calc_data puts -nonewline "\n    Element    Node(1)   Node(2)   Node(3)     Material"   
    
    set xp1 {container[@n="Properties"]/condition[@n="Shells"]/group}
    set xp2 ".//value\[@n='Material']"
    # We will save a TCL dictionary with the information from our materials
    set mat_dict ""
    set material_number 0 
    set formats ""
    
    foreach gNode [$root selectNodes $xp1] {
	
	set valueNode [$gNode selectNodes $xp2]
	set material_name [get_domnode_attribute $valueNode v] 
	
	if { ![dict exists $mat_dict $material_name] } {            
	    incr material_number
	    set mid $material_number
	    
	    #We also save the density of the material by checking the material database
	    set xp3 {/*/container[@n="Properties"]/container[@n="materials"]//}
	    append xp3 [format_xpath {blockdata[@n="Material" and @name=%s]/value[@n="Density"]} $material_name] 
	    set valueNode [$root selectNodes $xp3]
	    # All the introduced values are translated to 'm' and 'kg' with the help of this function
	    set density [gid_groups_conds::convert_value_to_default $valueNode]  
		        
	    dict set mat_dict $material_name MID $material_number 
	    dict set mat_dict $material_name DENSITY [format "%13.5E" $density] 
	     
	} else {
	    set mid [dict get $mat_dict $material_name MID]
	}
	
	set f {\n%10d %10d %10d %10d [format "%10d" $mid]}
	set f [subst $f]  
	dict set formats [$gNode @n] $f            
    } 
    
    if {$material_number=="0"} {
	error [= "Remember to define your surface/s as shell/s"]        
    }
    
    if {[write_calc_data has_elements -elemtype "Triangle" $formats]} {            
	write_calc_data connectivities -do_subst -elemtype "Triangle" $formats           
    } else {        
	error [= "Only Triangle elements are allowed in this version of the problemtype"]
    } 
    
    #################### MATERIALS ############################
       
    write_calc_data puts -nonewline "\n\n................................................................."
    write_calc_data puts -nonewline "\n\nBegin Materials"
    write_calc_data puts -nonewline "\nN Materials= $material_number"
    write_calc_data puts -nonewline "\nMat.          Density"
    write_calc_data puts -nonewline "\n................................................................."  
    
    # We print all the material data directly from the saved dictionary
    foreach material [dict keys $mat_dict] {
	write_calc_data puts -nonewline "\n[format "%4d" [dict get $mat_dict $material MID]] [dict get $mat_dict $material DENSITY]" 
    }
    
    #################### CONCENTRATE WEIGHTS ############################
    
    write_calc_data puts -nonewline "\n................................................................."  
    write_calc_data puts -nonewline "\nPoint Conditions" 
   
    set xp1 {condition[@n="Point_Weight"]/group}
    set xp2 {.//value[@n="Weight"]}   
    set formats ""
    
    foreach gNode [$root selectNodes $xp1] {        
	set valueNode [$gNode selectNodes $xp2]          
	set weight [gid_groups_conds::convert_value_to_default $valueNode] 
	set f {\n%d [format "%13.5E" $weight]}  
	set f [subst $f]        
	dict set formats [$gNode @n] $f  
    }

    write_calc_data puts -nonewline "\nConcentrate Weights"  
    write_calc_data puts -nonewline "\n[write_calc_data nodes -count $formats]" 
    write_calc_data puts -nonewline "\n................................................................." 
    write_calc_data puts -nonewline "\n\nPotencials Prescrits:"
    write_calc_data puts -nonewline "\n   Node  Tipus"
    write_calc_data puts -nonewline "\n   Valor/Etiqueta"
	
    write_calc_data nodes -do_subst $formats
    
    write_calc_data puts -nonewline "\n\n................................................................."  
}      

