# Ver TODO al final del archivo.

namespace eval ::PostGraphs {
  variable data
  variable cmds

  array set data {
    cmds,Point_Evolution pointevolution
    cmds,Point_Evolution,X,require 0
    cmds,Point_Evolution,Y,require 1
    cmds,Point_Graph pointgraph
    cmds,Point_Graph,X,require 2
    cmds,Point_Graph,Y,require 2
    cmds,Border_Graph bordergraph
    cmds,Border_Graph,X,require 1
    cmds,Border_Graph,Y,require 1
    cmds,Line_Graph linegraph
    cmds,Line_Graph,X,require 0
    cmds,Line_Graph,Y,require 1
    cmds,Integrate integrate
    cmds,Integrate,X,require 0
    cmds,Integrate,Y,require 1
    tmp,applying 0
    tmp,changename ""
  }

#   catch {
#     namespace forget ::GiDTk::*
#     namespace import ::GiDTk::*
#   }
}

proc ::PostGraphs::Create {{w .gid.graphs}} {
  variable data

  InitWindow $w [_ "Graphs Window"] PostResultsGraphsWindowGeom ::PostGraphs::Create
    if { ![winfo exists $w] } return ;# windows disabled || usemorewindows == 0
  set data(wid,main) $w

  set def_back [$w cget -background]
  set data(background) [ CCGetRGB $w $def_back]
  set data(activebg) [ CCColorActivo $def_back]

  set nb [NoteBook $w.nb -internalborderwidth 1 \
		 -activebackground $data(activebg) ]

  set data(wid,nb) $w.nb

  #pagina para seleccion de graphicas
  set fcreate [$nb insert end create -text [_ "Create"]]
  #pagina de opciones generales
  set fgoptions [$nb insert end goptions -text [_ "General Options"]]
  #pagina de opciones del tipo de coordenadas
  set fcoptions [$nb insert end coptions -text [_ "Coordinates Type"]]
  #pagina de opciones por linea
  set floptions [$nb insert end loptions -text [_ "Graph Management"] \
		     -raisecmd ::PostGraphs::KeepLBSelection]

  CreateGraphs $fcreate
  CreateGOptions $fgoptions
  CreateCOptions $fcoptions
  CreateLOptions $floptions

  set fcmd [frame $w.fcmd -background $data(activebg)]
  set data(wid,fcmd) $fcmd
  button $fcmd.bapply -text [_ "Apply"] -command ::PostGraphs::OnApply \
      -takefocus 0 -bd 1
  button $fcmd.breset -text [_ "Reset"] -command ::PostGraphs::OnReset \
      -takefocus 0 -bd 1
  set data(wid,breset) $fcmd.breset
  button $fcmd.bclose -text [_ "Close"] -command ::PostGraphs::OnClose \
      -takefocus 0 -bd 1

  button $w.toggle -image [GetImage "postgraf.gif"] \
      -command ::PostGraphs::ToggleView -takefocus 0 -bd 1
  GidHelpT $w.toggle [_ "Toggle mesh and graphs view"]
  catch {
    SetWidgetsWidthFromText $fcmd.bapply $fcmd.breset $fcmd.bclose
  }
  grid $fcmd.bapply $fcmd.breset $fcmd.bclose -padx 5 -pady 6
  $nb itemconfigure create -raisecmd "grid remove $fcmd.breset"
  $nb itemconfigure goptions -raisecmd "grid $fcmd.breset"
  $nb itemconfigure coptions -raisecmd "grid $fcmd.breset"
  $nb itemconfigure loptions -raisecmd "grid remove $fcmd.breset"

  grid $nb -sticky "snew" -columnspan 2
  grid $fcmd -sticky "snew"
  if { $::tcl_version >= 8.5 } { grid anchor $fcmd center }

  grid $w.toggle -row 1 -column 1 -sticky "e"

  grid columnconfigure $w 0 -weight 1
  grid rowconfigure $w 0 -weight 1
  $nb raise create

  focus $fcmd.bapply
  bind $w <Return> "tkButtonInvoke $fcmd.bapply"
  bind $w <Alt-c> "tkButtonInvoke $fcmd.bclose"
  bind $w <Escape> "tkButtonInvoke $fcmd.bclose"

  bind $w <${::acceleratorKey}-a> [bind .gid <${::acceleratorKey}-a>]
}

proc ::PostGraphs::CreateGraphs { f } {
  set fview [frame $f.fview]
  CreateView $fview
  set faxis [frame $f.faxis]
  CreateAxis $faxis
  UpdateResults
  grid $fview -row 0 -column 0 -sticky "snew" -pady 3
  grid $faxis -row 1 -column 0 -sticky "snew" -pady 3
  grid columnconfigure $f 0 -weight 1
  grid rowconfigure $f 1 -weight 1
}

proc ::PostGraphs::OnToggleTitlePosition { } {
  GiD_Process escape escape escape escape escape \
      results graphs optionsgraph legend2title
}

proc ::PostGraphs::PostInfo {idx args} {
  set post_info [eval GiD_Info postprocess get graphs_option $args]
  lindex [lindex $post_info 0] $idx
}

proc ::PostGraphs::CreateGOptions { f } {
  variable data

  # Outline on model
  label $f.loutline -text [_ "Outline on model"]:
  checkbutton $f.ckoutline -variable ::PostGraphs::data(opt,outline) -bd 1 \
      -onvalue "Yes" -offvalue "No"
  set data(wid,ckoutline) $f.cboutline

  # Grids
  label $f.lgrids -text [_ "Grids"]:
  checkbutton $f.ckgrids -variable ::PostGraphs::data(opt,grids) -bd 1 \
      -onvalue "Yes" -offvalue "No"
  set data(wid,ckgrids) $f.ckgrids

  # Current style
  set style_info [ GiD_Info postprocess get graphs_option CurrentStyle]
  set style_info [lindex $style_info 0]
  set data(opt,cstyle,all) [lindex $style_info 1]
  label $f.lcstyle -text [_ "Current style"]:
#   ComboBox $f.cbcstyle -width 10 -textvariable ::PostGraphs::data(opt,cstyle) \
#       -editable 0 -bd 1 \
#       -values [list [_ "Dot"] [_ "Line"] [_ "Dot Line"]]
  TTKComboBox $f.cbcstyle -width 10 \
      -textvariable ::PostGraphs::data(opt,cstyle) \
      -state readonly \
      -values {"Dot" "Line" "DotLine"} \
      -labels [list [_ "Dot"] [_ "Line"] [_ "Dot Line"]]
  $f.cbcstyle configure -editable 0
  set data(wid,cbcstyle) $f.cbcstyle

  # Title options
  TitleFrame $f.title -text [_ "Title"] -side left \
      -bd 2 -relief ridge
  set ftitle [$f.title getframe]
  label $ftitle.ltext -text [_ "Text"]:
  text $ftitle.ttext -width 20 -height 3
  set data(wid,ttext) $ftitle.ttext
  button $ftitle.btoggle -text [_ "Toggle position"] \
      -command ::PostGraphs::OnToggleTitlePosition
  grid $ftitle.ltext -sticky "sw"  -pady 2
  grid $ftitle.btoggle -sticky "se" -row 0 -column 3
  grid $ftitle.ttext -sticky "snew" -columnspan 4
  grid rowconfigure $ftitle 1 -weight 1
  grid columnconfigure $ftitle 3 -weight 1

  frame $f.axis

  # X-axis options
  TitleFrame $f.axis.x -text [_ "X axis"] -side left \
      -bd 2 -relief ridge
  set faxis [$f.axis.x getframe]
  CreateAxisOptions $faxis "X"

  # Y-axis options
  TitleFrame $f.axis.y -text [_ "Y axis"] -side left \
      -bd 2 -relief ridge
  set faxis [$f.axis.y getframe]
  CreateAxisOptions $faxis "Y"
  grid $f.axis.x $f.axis.y
  grid columnconfigure $f.axis 2 -weight 1

  grid $f.loutline $f.ckoutline -sticky "nw"
  grid $f.lgrids $f.ckgrids -sticky "nw"
  grid $f.lcstyle $f.cbcstyle -sticky "nw"
  grid $f.title -sticky "nwew" -columnspan 4
  grid $f.axis -sticky "nwew" -columnspan 4

  grid columnconfigure $f 3 -weight 1
  grid rowconfigure $f 10 -weight 1
  FillGOptions
}

proc ::PostGraphs::CreateAxisOptions {f axis}  {
  variable data
	
  label $f.lmin -text [_ "Minimun"]:
  entry $f.emin -textvariable ::PostGraphs::data(opt,axis,$axis,min)
  label $f.lmax -text [_ "Maximun"]:
  entry $f.emax -textvariable ::PostGraphs::data(opt,axis,$axis,max)
  label $f.ldiv -text [_ "Divisions"]:
  entry $f.ediv -textvariable ::PostGraphs::data(opt,axis,$axis,div)
  label $f.llabel -text [_ "Label"]:
  entry $f.elabel -textvariable ::PostGraphs::data(opt,axis,$axis,label)
  label $f.llog -text [_ "Logarithmic scale"]:
  checkbutton $f.elog -variable ::PostGraphs::data(opt,axis,$axis,log) -bd 1 -onvalue "Yes" -offvalue "No"
  #set data(wid,elog) $f.elog

  if { $axis eq "Y"} {
    set ::PostGraphs::ymin $f.emin
    set ::PostGraphs::ymax $f.emax
    set ::PostGraphs::ylog $f.elog
  }

  if { $axis eq "X"} {
    set ::PostGraphs::xmin $f.emin
    set ::PostGraphs::xmax $f.emax
    set ::PostGraphs::xlog $f.elog
  }

  grid $f.llog $f.elog  -sticky "nw"
  grid $f.lmin $f.emin  -sticky "nw"
  grid $f.lmax $f.emax  -sticky "nw"
  grid $f.ldiv $f.ediv  -sticky "nw"
  grid $f.llabel $f.elabel  -sticky "nw"

}

proc ::PostGraphs::FillGOptions { } {
  variable data

  # Outline on model
  set outline_info [ GiD_Info postprocess get graphs_option OutlineOnModel]
  set outline_info [lindex $outline_info 0]
  set data(opt,outline) [lindex $outline_info 0]

  # Grids
  set grids_info [ GiD_Info postprocess get graphs_option Grids]
  set grids_info [lindex $grids_info 0]
  set data(opt,grids) [lindex $grids_info 0]

  # Current style
  set style_info [ GiD_Info postprocess get graphs_option CurrentStyle]
  set style_info [lindex $style_info 0]
  set cur_sty [lindex $style_info 0]
  set cur_sty_idx [lsearch [lindex $style_info 1] $cur_sty]

    if { [Alive]} {
	if {$cur_sty_idx == -1} {
	    #$data(wid,cbcstyle) configure -text {}
	    $data(wid,cbcstyle) set ""
	} else {
	    #$data(wid,cbcstyle) setvalue "@$cur_sty_idx"
	    # $data(wid,cbcstyle) setvalue $cur_sty
	    $data(wid,cbcstyle) set $cur_sty
	}
    }

  # Title options
  set maintitle_info [ GiD_Info postprocess get graphs_option MainTitle]
  set maintitle_info [lindex $maintitle_info 0]

    if { [Alive]} {
	$data(wid,ttext) delete 1.0 end
	$data(wid,ttext) insert end $maintitle_info
    }

  # Axis options
  foreach a {X Y} {
    foreach {opt opt_arg} {log Log min Min max max div Divisions label label} {
      set axis_info [ GiD_Info postprocess get graphs_option ${a}_axis $opt_arg]
      set axis_info [lindex $axis_info 0]
      set data(opt,axis,$a,$opt) [lindex $axis_info 0]
    }
  }
}

proc ::PostGraphs::Alive { } {
  variable data

  expr {[info exists data(wid,main)] && [winfo exists $data(wid,main)]}
}

proc ::PostGraphs::UpdateColor {args} {
  variable data

  set color $data(opt,color,value)
  if {$color eq ""} {
    $data(wid,lcolorsel) configure -background $data(opt,nocolor)
    set data(opt,chkcolor,value) 0
  } else {
    $data(wid,lcolorsel) configure -background $color
    set data(opt,chkcolor,value) 1
  }
}

proc ::PostGraphs::UpdateOrigColor {args} {
  variable data

  set color $data(opt,origcolor,value)
  if {$color eq ""} {
    $data(wid,origcolor) configure -background $data(opt,nocolor)
  } else {
    $data(wid,origcolor) configure -background $color
  }
}

proc ::PostGraphs::UpdateRadColor {args} {
  variable data

  set color $data(opt,radcolor,value)
  if {$color eq ""} {
    $data(wid,radcolor) configure -background $data(opt,nocolor)
  } else {
    $data(wid,radcolor) configure -background $color
  }
}

proc ::PostGraphs::SelectColor {} {
  variable data

  if {[$data(wid,lcolorsel) cget -state] ne "disabled"} {
    if {$data(opt,color,value) eq ""} {
      set color [GIDChooseColor [winfo toplevel $data(wid,main)].selcolor \
		     -title [_ "Choose graph color"]]
    } else {
      set color [GIDChooseColor [winfo toplevel $data(wid,main)].selcolor \
		     -color $data(opt,color,value) \
		     -title [_ "Choose graph color"]]
    }

    if {$color ne ""} {
      set data(opt,color,value) $color
    }
  }
}

proc ::PostGraphs::SelectOrigColor {} {
  variable data

  if {[$data(wid,origcolor) cget -state] ne "disabled"} {
    if {$data(opt,origcolor,value) eq ""} {
      set color [GIDChooseColor [winfo toplevel $data(wid,main)].selcolor \
		     -title [_ "Choose origin axis color"]]
    } else {
      set color [GIDChooseColor [winfo toplevel $data(wid,main)].selcolor \
		     -color $data(opt,origcolor,value) \
		     -title [_ "Choose origin axis color"]]
    }

    if {$color ne ""} {
      set data(opt,origcolor,value) $color
    }
  }
}

proc ::PostGraphs::SelectRadColor {} {
  variable data

  if {[$data(wid,radcolor) cget -state] ne "disabled"} {
    if {$data(opt,radcolor,value) eq ""} {
      set color [GIDChooseColor [winfo toplevel $data(wid,main)].selcolor \
		     -title [_ "Choose radial marks color"]]
    } else {
      set color [GIDChooseColor [winfo toplevel $data(wid,main)].selcolor \
		     -color $data(opt,radcolor,value) \
		     -title [_ "Choose radial marks color"]]
    }

    if {$color ne ""} {
      set data(opt,radcolor,value) $color
    }
  }
}

proc ::PostGraphs::CBUpdateCurrent {cb {all ""} {cur ""}} {
  #set values [$cb cget -values]
  if {$cur eq ""} {
    #$cb configure -text {}
    $cb set ""
  } else {
#     set idx [lsearch $all $cur]
#     if {$idx == -1} {
#       lappend values $cur
#       set idx last
#     } else {
#       set idx "@$idx"
#     }
#     $cb setvalue "$idx"
#     $cb set $cur
    if { [ $cb current ] == -1 } {
      $cb configure -values [list {*}[ $cb cget -values ] $cur]
      $cb set $cur
    }
  }
}

proc ::PostGraphs::EntrySetValue { e v } {
  variable data

  $data(wid,$e) configure -validate none
  uplevel \#0 set [$data(wid,$e) cget -textvariable] $v
  $data(wid,$e) configure -validate all
}

proc ::PostGraphs::CreateCOptions { f } {
  variable data

  # Coordinates type
  trace remove variable ::PostGraphs::data(opt,coord) write "::PostGraphs::OnChangeCoord"

  set coord_info [ GiD_Info postprocess get graphs_option CoordType]
  set coord_info [lindex $coord_info 0]
  set data(opt,coord,all) [lindex $coord_info 1]
  label $f.lcoord -text [_ "Coordinates type"]:
  radiobutton $f.rcart -text [_ "Cartesian"] -value Cartesian -variable ::PostGraphs::data(opt,coord)
  radiobutton $f.rpolar -text [_ "Polar"] -value Polar -variable ::PostGraphs::data(opt,coord)

  trace add variable ::PostGraphs::data(opt,coord) write "::PostGraphs::OnChangeCoord"

  # Polar coord options
  TitleFrame $f.polar -text [_ "Polar coordinates"] -side left \
      -bd 2 -relief ridge
  set fpolar [$f.polar getframe]

  trace remove variable ::PostGraphs::data(opt,origcolor,value) write "::PostGraphs::UpdateOrigColor"
  trace remove variable ::PostGraphs::data(opt,radcolor,value) write "::PostGraphs::UpdateRadColor"

  label $fpolar.luse -text [_ "Use"]
  #ComboBox $fpolar.cbangleaxis -values [list x y] -textvariable ::PostGraphs::data(opt,angleaxis) -editable 0 -bd 1
  TTKComboBox $fpolar.cbangleaxis -values [list x y] \
      -textvariable ::PostGraphs::data(opt,angleaxis) -state readonly

  set data(wid,cbangleaxis) $fpolar.cbangleaxis

  label $fpolar.laxis -text [_ "axis values as angle"]
  label $fpolar.luse2 -text [_ "Use"]
  #ComboBox $fpolar.cbangleunit -values [list [_ "Degrees"] [_ "Radians"]] -textvariable ::PostGraphs::data(opt,angleunit) -editable 0 -bd 1

  TTKComboBox $fpolar.cbangleunit -values [list [_ "Degrees"] [_ "Radians"]] \
      -textvariable ::PostGraphs::data(opt,angleunit) -state readonly

  set data(wid,cbangleunit) $fpolar.cbangleunit

  label $fpolar.lunit -text [_ "as angle unit"]
  label $fpolar.laxes -text [_ "Show axes in (0,0) point"]:
  checkbutton $fpolar.ckaxes -variable ::PostGraphs::data(opt,origaxes) -bd 1 \
      -onvalue "Yes" -offvalue "No"
  label $fpolar.lorigaxes -text [_ "Show radial marks"]:
  checkbutton $fpolar.ckrad -variable ::PostGraphs::data(opt,radmarks) -bd 1 \
      -onvalue "Yes" -offvalue "No"

  #ComboBox $fpolar.cbradpat -textvariable ::PostGraphs::data(opt,patradmarks) -width 10 \
      #-editable 0 -bd 1 -values [list [_ "Continous"] [_ "Point"] [_ "Slash"]]
  TTKComboBox $fpolar.cbradpat \
      -textvariable ::PostGraphs::data(opt,patradmarks) -width 10 \
      -state readonly -values [ list "Continous" "Point" "Slash"] \
      -labels [ list [_ "Continous"] [_ "Point"] [_ "Slash"]]

  label $fpolar.origcolor -width 4 -bd 1 -relief solid
  bind $fpolar.origcolor <1> "PostGraphs::SelectOrigColor"
  set data(wid,origcolor) $fpolar.origcolor

  label $fpolar.radcolor -width 4 -bd 1 -relief solid
  bind $fpolar.radcolor <1> "PostGraphs::SelectRadColor"
  set data(wid,radcolor) $fpolar.radcolor
  set data(wid,cbradpat) $fpolar.cbradpat

  set data(opt,nocolor) [$fpolar.origcolor cget -background]

  trace add variable ::PostGraphs::data(opt,radcolor,value) write "::PostGraphs::UpdateRadColor"
  trace add variable ::PostGraphs::data(opt,origcolor,value) write "::PostGraphs::UpdateOrigColor"


  set ::PostGraphs::angleaxis $fpolar.cbangleaxis
  set ::PostGraphs::angleunit $fpolar.cbangleunit
  set ::PostGraphs::origaxes $fpolar.ckaxes
  set ::PostGraphs::radmarks $fpolar.ckrad
  set ::PostGraphs::radpattern $fpolar.cbradpat

  grid $fpolar.luse $fpolar.cbangleaxis $fpolar.laxis -sticky "nw"
  grid $fpolar.luse2 $fpolar.cbangleunit $fpolar.lunit -sticky "nw"
  grid $fpolar.origcolor $fpolar.laxes $fpolar.ckaxes -sticky "nw"
  grid $fpolar.radcolor $fpolar.lorigaxes $fpolar.ckrad $fpolar.cbradpat -sticky "nw"
  grid rowconfigure $fpolar 1 -weight 1
  grid columnconfigure $fpolar 3 -weight 1

    grid $f.rcart $f.rpolar
    grid $f.polar -sticky "nwew" -columnspan 4

    FillCOptions
}

proc ::PostGraphs::FillCOptions { } {
  variable data

  # coordinates type
  set coord_info [ GiD_Info postprocess get graphs_option CoordType]
  set coord_info [lindex $coord_info 0]
  set cur_coord [lindex $coord_info 0]
  set data(opt,coord) $cur_coord

  # Polar coordinates options
  set angaxis_info [ GiD_Info postprocess get graphs_option AngleAxis]
  set angaxis_info [lindex $angaxis_info 0]
  set cur_angaxis [lindex $angaxis_info 0]
  set cur_angaxis_idx [lsearch [lindex $angaxis_info 1] $cur_angaxis]
  if { [Alive]} {
     if {$cur_angaxis_idx == -1} {
	#$data(wid,cbangleaxis) configure -text {}
	$data(wid,cbangleaxis) set ""
     } else {
       #$data(wid,cbangleaxis) setvalue "@$cur_angaxis_idx"
       $data(wid,cbangleaxis) \
	   set [ lindex [ $data(wid,cbangleaxis) cget -values ] \
		     $cur_angaxis_idx ]
     }
  }

  set angunit_info [ GiD_Info postprocess get graphs_option AngleUnit]
  set angunit_info [lindex $angunit_info 0]
  set cur_angunit [lindex $angunit_info 0]
  set cur_angunit_idx [lsearch [lindex $angunit_info 1] $cur_angunit]
  if { [Alive]} {
      if {$cur_angunit_idx == -1} {
	#data(wid,cbangleunit) configure -text {}
	data(wid,cbangleunit) set ""
     } else {
	#$data(wid,cbangleunit) setvalue "@$cur_angunit_idx"
	$data(wid,cbangleunit) \
	    set [ lindex [ $data(wid,cbangleunit) cget -values ] \
		      $cur_angunit_idx ]
     }
  }

  set origaxes_info [ GiD_Info postprocess get graphs_option ShowOrigAxes]
  set origaxes_info [lindex $origaxes_info 0]
  set data(opt,origaxes) [lindex $origaxes_info 0]

  set radmarks_info [ GiD_Info postprocess get graphs_option ShowRadMarks]
  set radmarks_info [lindex $radmarks_info 0]
  set data(opt,radmarks) [lindex $radmarks_info 0]

  set origcolor_info [ GiD_Info postprocess get graphs_option ColorOrig]
  set origcolor_info [lindex $origcolor_info 0]
  set data(opt,origcolor,value) [lindex $origcolor_info 0]

  set radcolor_info [ GiD_Info postprocess get graphs_option ColorRad]
  set radcolor_info [lindex $radcolor_info 0]
  set data(opt,radcolor,value) [lindex $radcolor_info 0]

  set radpat_info [ GiD_Info postprocess get graphs_option PatRad]
  set radpat_info [lindex $radpat_info 0]
  set data(opt,patradmarks) [lindex $radpat_info 0]
  #set cur_radpat_idx [lsearch [lindex $radpat_info 1] $cur_radpat]


  if { [Alive]} {
     #if {$cur_radpat_idx == -1} {
	#$data(wid,cbradpat) configure -text {}
      #  $data(wid,cbradpat) set ""
      #} else {
	#$data(wid,cbradpat) setvalue "@$cur_radpat_idx"
       # $data(wid,cbradpat) set [lindex [$data(wid,cbradpat) cget -values ] $cur_radpat_idx]
      #}
  }

  OnChangeCoord

}


proc ::PostGraphs::OnChangeLSelection { } {
  variable data

  set cur_sel [$data(wid,lblines) curselection]
  set len_sel [llength $cur_sel]
  if {!$len_sel} {
    # Style
    #$data(wid,cbstyle) configure -text {}
    $data(wid,cbstyle) set ""

    # Color
    set data(opt,color,value) ""

    # remove checkbuttons
    foreach _w {chkstyle chkcolor chkcfill chklwidth chkpat chkpatfact \
		    chkpsize chklname} {
      grid remove $data(wid,$_w)
    }

    # disable widgets related
    foreach _w {bdel lstyle cbstyle lcolor ecolor lcolorsel cbcfill \
		    llwidth sclwidth lpat cbpat lpatfact epatfact \
		    lpsize scpsize llname elname} {
      $data(wid,$_w) configure -state disabled
    }
  } elseif {$cur_sel != $data(opt,lines,cur_sel)} {
    foreach _w {lstyle cbstyle bdel lcolor ecolor lcolorsel cbcfill \
		    llwidth sclwidth lpat cbpat lpatfact epatfact \
		    lpsize scpsize llname elname} {
      $data(wid,$_w) configure -state normal
    }
    set data(opt,lines,cur_sel) $cur_sel
    if {$len_sel>1} {
      # restore checkbuttons
      foreach _w {chkstyle  chkcfill chklwidth chkpat chkpatfact \
		      chkpsize chklname} {
	set data(opt,$_w,value) 0
	grid $data(wid,$_w)
      }
      # Style
      CBUpdateCurrent $data(wid,cbstyle)

      # Color
      set data(opt,color,value) ""
      # LineWidth

      # Pattern
      CBUpdateCurrent $data(wid,cbpat)

      # PatternFactor

      # PointSize

      # Title
      set data(opt,elname,value) ""
      foreach _w {chklname llname elname} {
	$data(wid,$_w) configure -state disabled
      }
   } else {
     # remove checkbuttons
     foreach _w {chkstyle chkcolor chkcfill chklwidth chkpat chkpatfact \
		     chkpsize chklname} {
       set data(opt,$_w,value) 0
       # grid $data(wid,$_w)
       grid remove $data(wid,$_w)
     }
     set g [lindex $data(opt,lines,all) $cur_sel]
     # Style
     set style [lindex [ GiD_Info postprocess get graphs_option style $g] 0]
     set style [lindex $style 0]
     CBUpdateCurrent $data(wid,cbstyle) $data(opt,style,all,names) $style
	
     # CFill
     set cfill [lindex [ GiD_Info postprocess get graphs_option colorascfill $g] 0]
     set cfill [lindex $cfill 0]
     set data(opt,cfill,value) $cfill

     # Color
     if { $data(opt,cfill,value) eq "No" } {
	 set data(opt,chkcolor,value) 0
	 $data(wid,lcolor) configure -state normal
	 $data(wid,ecolor) configure -state normal
	 $data(wid,lcolorsel) configure -state normal
	 set color [lindex [ GiD_Info postprocess get graphs_option color $g] 0]
     } else {
	 set data(opt,chkcolor,value) 1
	 $data(wid,lcolor) configure -state disabled
	 $data(wid,ecolor) configure -state disabled
	 $data(wid,lcolorsel) configure -state disabled
	 set color ""
     }
     #set color [lindex [ GiD_Info postprocess get graphs_option color $g] 0]
     set data(opt,color,value) $color

     # LineWidth
     set lw [lindex [ GiD_Info postprocess get graphs_option linewidth $g] 0]
     set data(opt,lwidth,value) [lindex $lw 0]

     # Pattern
     set cur_pat_val [lindex [ GiD_Info postprocess get graphs_option pattern $g] 0]
     set cur_pat_val [string range [lindex $cur_pat_val 0] 1 end]
     set all [list ]
     set cur_pat_str ""
     foreach pat $data(opt,pattern,all,names) {
       set str $data(opt,pattern,$pat,str)
       lappend all $str
       if {$data(opt,pattern,$pat,val) == $cur_pat_val} {
	 set cur_pat_str $str
       }
     }
     if {$cur_pat_str eq ""} {
       set cur_pat_str $cur_pat_val
     }
     CBUpdateCurrent $data(wid,cbpat) $all $cur_pat_str

     # PatternFactor
     set patfact [lindex [ GiD_Info postprocess get graphs_option patternfactor $g] 0]
     set patfact [lindex $patfact 0]
     EntrySetValue epatfact $patfact

     # PointSize
     set lw [lindex [ GiD_Info postprocess get graphs_option pointsize $g] 0]
     set data(opt,psize,value) [lindex $lw 0]

     # Title
     set data(opt,lname,value) $g
     foreach _w {llname elname} {
       grid $data(wid,$_w)
     }
   }
  }
}

proc ::PostGraphs::KeepLBSelection { } {
    variable data

    if { ![winfo exists $data(wid,nb)] } return

    if {[$data(wid,nb) raise] eq "loptions"} {
	foreach idx $data(opt,lines,cur_sel) {
	    $data(wid,lblines) selection set $idx
	}
    }
}

proc ::PostGraphs::LO_OnModifyCB { name } {
  variable data

  set data(opt,chk$name,value) 1
  after idle ::PostGraphs::KeepLBSelection

}

proc ::PostGraphs::CreateLOptions { f } {
  variable data

  panedwindow $f.pn -orient horizontal
  LabelFrame $f.pn.g -side top -text [_ "Graphs"]: -anchor w
  LabelFrame $f.pn.o -side top -text [_ "Options"]: -anchor w
  $f.pn add $f.pn.g $f.pn.o -sticky "snew"

  # Graphs list frame
  set fg [$f.pn.g getframe]
  listbox $fg.lb -bd 1 -selectmode extended
  set data(wid,lblines) $fg.lb
  bind $fg.lb <<ListboxSelect>> +::PostGraphs::OnChangeLSelection
  bind $fg.lb <Delete> ::PostGraphs::DeleteLines

  button $fg.bdel -image [GetImage redcross22x22.gif] -text [_ "Delete"] \
      -compound left -command ::PostGraphs::DeleteLines
  set data(wid,bdel) $fg.bdel
  grid $fg.lb -sticky "snew"
  grid $fg.bdel
  grid rowconfigure $fg 0 -weight 1
  grid columnconfigure $fg 0 -weight 1

  # Graphs options frame
  set fo [$f.pn.o getframe]

  # Style
  set data(opt,style,all,names) {None Dot Line DotLine}
  set values [list [_ "None"] [_ "Dot"] \
		  [_ "Line"] [_ "Dot Line"]]
  checkbutton $fo.chkstyle -bd 1 \
      -variable ::PostGraphs::data(opt,chkstyle,value)
  label $fo.lstyle -text [_ "Style"]:
  #ComboBox $fo.cbstyle -editable 0 -bd 1 \
  #    -values $values -textvariable ::PostGraphs::data(opt,style,value) \
  #    -modifycmd "::PostGraphs::LO_OnModifyCB style"
  TTKComboBox $fo.cbstyle -state readonly \
      -values $values -textvariable ::PostGraphs::data(opt,style,value) \
      -modifycmd "::PostGraphs::LO_OnModifyCB style"
  set data(wid,chkstyle) $fo.chkstyle
  set data(wid,lstyle) $fo.lstyle
  set data(wid,cbstyle) $fo.cbstyle

  # Color
  checkbutton $fo.chkcolor -bd 1 \
      -variable ::PostGraphs::data(opt,chkcolor,value)
  label $fo.lcolor -text [_ "Color"]:
  trace remove variable ::PostGraphs::data(opt,color,value) write "::PostGraphs::UpdateColor"
  frame $fo.fc
  entry $fo.fc.ecolor -textvariable ::PostGraphs::data(opt,color,value)
  label $fo.fc.lcolorsel -width 4 -bd 1 -relief solid
  bind $fo.fc.lcolorsel <1> "PostGraphs::SelectColor"
  set data(wid,chkcolor) $fo.chkcolor
  set data(wid,lcolor) $fo.lcolor
  set data(wid,ecolor) $fo.fc.ecolor
  set data(wid,lcolorsel) $fo.fc.lcolorsel
  set data(opt,nocolor) [$fo.fc.lcolorsel cget -background]
  grid $fo.fc.ecolor $fo.fc.lcolorsel -sticky "ew"
  grid columnconfigure $fo.fc 0 -weight 1
  trace add variable ::PostGraphs::data(opt,color,value) write "::PostGraphs::UpdateColor"

  # Color as contour fill

  #frame $fo.fcf
  checkbutton $fo.chkcfill -bd 1 -variable ::PostGraphs::data(opt,chkcfill,value)
  label $fo.lcfill -text ""
  trace remove variable ::PostGraphs::data(opt,cfill,value) write "::PostGraphs::OnChangeCFill"
  checkbutton $fo.cbcfill -bd 1 \
	-variable ::PostGraphs::data(opt,cfill,value) -text [_ "Color as contour fill"] -onvalue "Yes" -offvalue "No"

  #bind $fo.cbcfill <1>  "::PostGraphs::OnChangeLSelection"

  set data(wid,chkcfill) $fo.chkcfill
  set data(wid,cbcfill) $fo.cbcfill
  set data(opt,nocolor) [$fo.fc.lcolorsel cget -background]

  trace add variable ::PostGraphs::data(opt,cfill,value) write "::PostGraphs::OnChangeCFill"


  # LineWidth
  checkbutton $fo.chklwidth  -bd 1 \
      -variable ::PostGraphs::data(opt,chklwidth,value)
  label $fo.llwidth -text [_ "Line width"]:
  scale $fo.sclwidth -showvalue yes -bd 1 \
      -orient horizontal -variable ::PostGraphs::data(opt,lwidth,value) \
      -command "::PostGraphs::OnChangeScale lwidth"
  set data(wid,chklwidth) $fo.chklwidth
  set data(wid,llwidth) $fo.llwidth
  set data(wid,sclwidth) $fo.sclwidth

  # Pattern
  set dot [ GiDTeXSymbols::Get {\mult}]
  array set data [list \
		      opt,pattern,Continuous,str "--------" \
		      opt,pattern,Continuous,val ffff \
		      opt,pattern,Slash,str "--  --  " \
		      opt,pattern,Slash,val ff00 \
		      opt,pattern,SlashDot,str "- $dot - $dot " \
		      opt,pattern,SlashDot,val ff18 \
		      opt,pattern,LittleSlash,str "- - - - " \
		      opt,pattern,LittleSlash,val f0f0 \
		      opt,pattern,Dots,str "$dot $dot $dot $dot " \
		      opt,pattern,Dots,val f0f0 \
		      opt,pattern,BigSlash,str "--- --- " \
		      opt,pattern,BigSlash,val cccc \
		      opt,pattern,SlashSpace,str "-   -   " \
		      opt,pattern,SlashSpace,val f000 \
		      opt,pattern,DotSlashDot,str "$dot-$dot$dot-$dot" \
		      opt,pattern,DotSlashDot,val cfcc]
  set data(opt,pattern,all,names) \
      {Continuous Slash SlashDot LittleSlash Dots BigSlash SlashSpace DotSlashDot}
  set all [list ]
  foreach pat $data(opt,pattern,all,names) {
    lappend all $data(opt,pattern,$pat,str)
  }
  checkbutton $fo.chkpat  -bd 1 \
      -variable ::PostGraphs::data(opt,chkpat,value)
  label $fo.lpat -text [_ "Pattern"]:
  #ComboBox $fo.cbpat -editable 0 -bd 1 \
  #    -values $all -textvariable ::PostGraphs::data(opt,pat,value) \
  #    -modifycmd "::PostGraphs::LO_OnModifyCB pat"
  TTKComboBox $fo.cbpat -state readonly \
      -values $all -textvariable ::PostGraphs::data(opt,pat,value) \
      -modifycmd "::PostGraphs::LO_OnModifyCB pat"
  set data(wid,chkpat) $fo.chkpat
  set data(wid,lpat) $fo.lpat
  set data(wid,cbpat) $fo.cbpat

  # PatternFactor
  checkbutton $fo.chkpatfact  -bd 1 \
      -variable ::PostGraphs::data(opt,chkpatfact,value)
  label $fo.lpatfact -text [_ "Pattern factor"]:
  entry $fo.epatfact -bd 1 \
      -textvariable ::PostGraphs::data(opt,patfact,value) \
      -validate all \
      -vcmd "::PostGraphs::OnEditLPatFact %V %P %s"
  set data(wid,chkpatfact) $fo.chkpatfact
  set data(wid,lpatfact) $fo.lpatfact
  set data(wid,epatfact) $fo.epatfact

  # PointSize
  checkbutton $fo.chkpsize -bd 1  \
      -variable ::PostGraphs::data(opt,chkpsize,value)
  label $fo.lpsize -text [_ "Point size"]:
  scale $fo.scpsize -showvalue yes -bd 1 \
      -orient horizontal -variable ::PostGraphs::data(opt,psize,value) \
      -command "::PostGraphs::OnChangeScale psize"
  set data(wid,chkpsize) $fo.chkpsize
  set data(wid,lpsize) $fo.lpsize
  set data(wid,scpsize) $fo.scpsize

  # InvertLineGraph

  # Title
  checkbutton $fo.chklname -bd 1 \
      -variable ::PostGraphs::data(opt,chklname,value)
  label $fo.llname -text [_ "Name"]:
  set data(wid,llname) $fo.llname
  entry $fo.elname -bd 1 \
      -textvariable ::PostGraphs::data(opt,lname,value) \
      -validate all \
      -vcmd "::PostGraphs::OnEditLName %V %P %s"
  set data(wid,chklname) $fo.chklname
  set data(wid,llname) $fo.llname
  set data(wid,elname) $fo.elname

  set row 0
  foreach {w1 w2 w3} {chkstyle lstyle cbstyle \
		       chkcolor lcolor fc \
		       chkcfill lcfill cbcfill \
		       chklwidth llwidth sclwidth \
		       chkpat lpat cbpat \
		       chkpatfact lpatfact epatfact \
		       chkpsize lpsize scpsize \
		       chklname llname elname} {
    grid $fo.$w1 -sticky "s" -row $row -column 0 -pady 2
    grid $fo.$w2 -sticky "sw" -row $row -column 1 -pady 2
    grid $fo.$w3 -sticky "sew" -row $row -column 2 -pady 2
    incr row
  }

  grid columnconfigure $fo 2 -weight 1
  grid rowconfigure $fo $row -weight 10
  grid columnconfigure $fo 3 -weight 1

  grid $f.pn -sticky "snew"
  grid rowconfigure $f 0 -weight 1
  grid columnconfigure $f 0 -weight 1
  set data(opt,lines,need_conf) 1
  set data(opt,lines,cur_sel) {}
  FillLOptions
  bind $data(wid,main) <Any-Key> +[list after idle ::PostGraphs::KeepLBSelection]
  bind $data(wid,main) <FocusIn> +[list after idle ::PostGraphs::KeepLBSelection]
}

proc ::PostGraphs::OnChangeScale { name v } {
  variable data

  set data(opt,chk$name,value) 1
}

proc ::PostGraphs::OnChangeCFill {args} {
  variable data

  set data(opt,chkcfill,value) 1


  if { $data(opt,cfill,value) eq "No" } {
	set data(opt,chkcolor,value) 0
	$data(wid,lcolor) configure -state normal
	$data(wid,ecolor) configure -state normal
	$data(wid,lcolorsel) configure -state normal
	set cur_sel [$data(wid,lblines) curselection]
	set g [lindex $data(opt,lines,all) $cur_sel]
	set color [lindex [ GiD_Info postprocess get graphs_option color $g] 0]
	set data(opt,color,value) $color
  } else {
	set data(opt,chkcolor,value) 1
	$data(wid,lcolor) configure -state disabled
	$data(wid,ecolor) configure -state disabled
	$data(wid,lcolorsel) configure -state disabled
	set data(opt,color,value) ""
  }

}

proc ::PostGraphs::OnChangeCoord { args } {
    variable data

    if { $::PostGraphs::data(opt,coord) eq "Polar"} {
	#$::PostGraphs::xmin configure -state disabled
	#$::PostGraphs::xmax configure -state disabled
	$::PostGraphs::xlog configure -state disabled
	
	#$::PostGraphs::ymin configure -state disabled
	#$::PostGraphs::ymax configure -state disabled
	$::PostGraphs::ylog configure -state disabled
	
	$::PostGraphs::angleaxis configure -state normal
	$::PostGraphs::angleunit configure -state normal
	$::PostGraphs::origaxes configure -state normal
	$::PostGraphs::radmarks configure -state normal
	$::PostGraphs::radpattern configure -state normal
	
	$data(wid,origcolor) configure -state normal
	$data(wid,radcolor) configure -state normal
	set origcolor_info [ GiD_Info postprocess get graphs_option ColorOrig]
	set origcolor_info [lindex $origcolor_info 0]
	set data(opt,origcolor,value) [lindex $origcolor_info 0]

	set radcolor_info [ GiD_Info postprocess get graphs_option ColorRad]
	set radcolor_info [lindex $radcolor_info 0]
	set data(opt,radcolor,value) [lindex $radcolor_info 0]
	UpdateOrigColor
	UpdateRadColor
    }
    
    if { $::PostGraphs::data(opt,coord) eq "Cartesian"} {
	$::PostGraphs::xmin configure -state normal
	$::PostGraphs::xmax configure -state normal
	$::PostGraphs::xlog configure -state normal
	
	$::PostGraphs::ymin configure -state normal
	$::PostGraphs::ymax configure -state normal
	$::PostGraphs::ylog configure -state normal
	
	$::PostGraphs::angleaxis configure -state disabled
	$::PostGraphs::angleunit configure -state disabled
	$::PostGraphs::origaxes configure -state disabled
	$::PostGraphs::radmarks configure -state disabled
	$::PostGraphs::radpattern configure -state disabled
	
	$data(wid,origcolor) configure -state disabled
	$data(wid,radcolor) configure -state disabled
	set data(opt,origcolor,value) ""
	set data(opt,radcolor,value) ""
	UpdateOrigColor
	UpdateRadColor
    }
}

proc ::PostGraphs::OnEditLName { action new old } {
  variable data

  if {$action eq "key" } {
    set data(opt,chklname,value) 1
  } elseif {$action eq "focusin"} {
    set data(opt,lname,value,save) $old
  } elseif {$action eq "focusout"} {
    if {$new eq ""} {
      set data(opt,lname,value) $data(opt,lname,value,save)
    }
  }
  after idle $data(wid,elname) configure -validate all
  return 1
}

proc ::PostGraphs::OnEditLPatFact { action new old } {
  variable data

  if {$action eq "key" } {
    set data(opt,chkpatfact,value) 1
  } elseif {$action eq "focusin"} {
    set data(opt,patfact,value,save) $old
  } elseif {$action eq "focusout"}  {
    if {(![string is double $new] || $new<0)} {
      EntrySetValue epatfact $data(opt,patfact,value,save)
    }
  }
  after idle $data(wid,epatfact) configure -validate all
  return 1
}

proc ::PostGraphs::DeleteLines {  } {
  variable data

  set data(tmp,applying) 1
  foreach idx [$data(wid,lblines) cursel] {
    GiD_Process escape escape escape escape Results Graphs OptionsGraph \
	DeleteGraph [$data(wid,lblines) get $idx]
  }
  FillLOptions
  set data(tmp,applying) 0
}

proc ::PostGraphs::FillLOptions { } {
  variable data

  if {[Alive]} {
    set all [ GiD_Info postprocess get all_graphs]
    set prevstate [$data(wid,lblines) cget -state]
    $data(wid,lblines) configure -state normal
    set all_lb [$data(wid,lblines) get 0 end]
    # update new selection names
    set next_selection {}
    foreach idx $data(opt,lines,cur_sel) {
      if {[set idx1 [lsearch $all [lindex $all_lb $idx]]]!=-1} {
	lappend next_selection $idx1
      }
    }
    $data(wid,lblines) delete 0 end
    set data(opt,lines,all) $all
    if {$all ne ""} {
      eval $data(wid,lblines) insert end $all
      if {[llength $next_selection]} {
	eval $data(wid,lblines) selection set $next_selection
      } elseif {$data(tmp,changename) ne ""} {
	set idx [lsearch $all $data(tmp,changename)]
	if {$idx == -1} {
	  WarnWinText "$data(tmp,changename) should be a Graph name"
	} else {
	  $data(wid,lblines) selection set $idx
	}
      }
      if {$data(opt,lines,need_conf)} {
	set first_line [lindex $all 0]
	set data(opt,lines,need_conf) 0
	
	# LineWidth
	set lw_info [PostInfo 1 linewidth $first_line]
	foreach {lw_from lw_to lw_step} $lw_info break
	$data(wid,sclwidth) configure \
	    -from $lw_from -to $lw_to -resolution $lw_step
	
	# PointSize
	set ps_info [PostInfo 1 pointsize $first_line]
	foreach {ps_from ps_to ps_step} $ps_info break
	$data(wid,scpsize) configure \
	    -from $ps_from -to $ps_to -resolution $ps_step
      }
    }
    set data(opt,lines,cur_sel) {}
    $data(wid,lblines) configure -state $prevstate
    OnChangeLSelection
  }
}

proc ::PostGraphs::OnReset { } {
  variable data

  # Reset Options

  # Coordinates type
  GiD_Process Mescape Results Graphs OptionsGraph Coordtype Cartesian escape
  OnChangeCoord

  # Outline on model

  # Grids

  # Style

  # MainTitle
  GiD_Process escape escape escape escape Results Graphs OptionsGraph MainTitle ''

  # Axis
  GiD_Process X_Axis Reset Y_Axis Reset

  FillGOptions
  FillCOptions
}

proc ::PostGraphs::OnApply { } {
  variable data
  switch [$data(wid,nb) raise] {
    create {
      ApplyCreate
    }
    goptions {
      ApplyGOptions
    }
    coptions {
      ApplyCOptions
    }
    loptions {
      ApplyLOptions
    }
  }
}

proc ::PostGraphs::ApplyGOptions { } {
  variable data

  set data(tmp,applying) 1

  # Outline on model
  set cur [lindex [lindex [ GiD_Info postprocess get graphs_option OutlineOnModel] 0] 0]
  if {$cur ne $data(opt,outline)} {
    GiD_Process escape escape escape escape \
	Results Graphs OptionsGraph GraphOutLineOnModel \
	[expr {$data(opt,outline)?"Yes":"No"}]
  }

  # Grids
  set cur [lindex [lindex [ GiD_Info postprocess get graphs_option Grids] 0] 0]
  if {$cur ne $data(opt,grids)} {
    GiD_Process escape escape escape escape \
	Results Graphs OptionsGraph Grids \
	[expr {$data(opt,grids)?"Yes":"No"}]
  }

  # Current style
  #set idx [$data(wid,cbcstyle) getvalue]
  set idx [ $data(wid,cbcstyle) current ]
  if {$idx != -1} {
    set style_info [lindex [ GiD_Info postprocess get graphs_option CurrentStyle] 0]
    set all_styles [lindex $style_info 1]
    set cur_sty_idx [lsearch $all_styles [lindex $style_info 0]]
    if {$cur_sty_idx != $idx} {
    GiD_Process escape escape escape escape \
	Results Graphs OptionsGraph CurrentStyle \
	[lindex $all_styles $idx]
    }
  }

  # MainTitle
  set str_title [string trim [$data(wid,ttext) get 1.0 end]]
  GiD_Process escape escape escape escape \
      Results Graphs OptionsGraph MainTitle $str_title

   # ApplyCOptions


  # Axis options
  foreach a {X Y} {
    foreach {opt opt_arg} {log Log min Min max max div Divisions label Label} {
      set axis_info [ GiD_Info postprocess get graphs_option ${a}_axis $opt_arg]
      set axis_info [lindex $axis_info 0]
      set coordtype [lindex [ GiD_Info postprocess get graphs_option CoordType] 0]
      if {$data(opt,axis,$a,$opt) ne [lindex $axis_info 0] || $coordtype eq "Polar"} {
	    GiD_Process escape escape escape escape \
	    Results Graphs OptionsGraph ${a}_axis \
	    Set${opt_arg} $data(opt,axis,$a,$opt)

      }
    }
  }


  #ApplyCOptions
  set data(tmp,applying) 0
  FillGOptions
}

proc ::PostGraphs::ApplyCOptions { } {
  variable data

  set data(tmp,applying) 1

  # Polar coordiantes options
  #set idx [$data(wid,cbangleaxis) getvalue]
  set idx [ $data(wid,cbangleaxis) current ]
  if {$idx != -1} {
    set angleaxis_info [lindex [ GiD_Info postprocess get graphs_option AngleAxis] 0]
    set all_axis [lindex $angleaxis_info 1]
    set cur_angleaxis_idx [lsearch $all_axis [lindex $angleaxis_info 0]]
	if {$cur_angleaxis_idx != $idx} {
      GiD_Process Mescape Results Graphs OptionsGraph CoordType Options AngleAxis [lindex $all_axis $idx] escape
    }
  }

  #set idx [$data(wid,cbangleunit) getvalue]
  set idx [ $data(wid,cbangleunit) current ]
  if {$idx != -1} {
    set angleunit_info [lindex [ GiD_Info postprocess get graphs_option AngleUnit] 0]
    set all_unit [lindex $angleunit_info 1]
    set cur_angleunit_idx [lsearch $all_unit [lindex $angleunit_info 0]]
	if {$cur_angleunit_idx != $idx} {
      GiD_Process Mescape Results Graphs OptionsGraph CoordType Options AngleUnit [lindex $all_unit $idx] escape
    }
  }

  set cur [lindex [lindex [ GiD_Info postprocess get graphs_option ShowOrigAxes] 0] 0]
  if {$cur ne $data(opt,origaxes)} {
    GiD_Process Mescape Results Graphs OptionsGraph CoordType Options ShowOrigAxes \
	[expr {$data(opt,origaxes)?"Yes":"No"}]
  }

  set cur [lindex [lindex [ GiD_Info postprocess get graphs_option ShowRadMarks] 0] 0]
  if {$cur ne $data(opt,radmarks)} {
    GiD_Process Mescape Results Graphs OptionsGraph CoordType Options ShowRadMarks \
	[expr {$data(opt,radmarks)?"Yes":"No"}]
  }

  set origcolor [lindex [ GiD_Info postprocess get graphs_option ColorOrig] 0]
  if {$data(opt,origcolor,value) != "" && $origcolor ne $data(opt,origcolor,value)} {
    GiD_Process Mescape Results Graphs OptionsGraph CoordType Options ColorOrigAxes $data(opt,origcolor,value)
  }
	
  set radcolor [lindex [ GiD_Info postprocess get graphs_option ColorRad] 0]
  if {$data(opt,radcolor,value) != "" && $radcolor ne $data(opt,radcolor,value)} {
    GiD_Process Mescape Results Graphs OptionsGraph CoordType Options ColorRadMarks $data(opt,radcolor,value)
  }

  #set idx [$data(wid,cbradpat) cget -values]
  #if {$idx != -1} {
  #      set radpat [lindex [ GiD_Info postprocess get graphs_option PatRad] 0]
  #      set all_radpat [lindex $radpat 0]
  #      set cur_radpat_idx [lsearch $all_radpat [lindex $radpat 0]]
  #      if {$cur_radpat_idx != $idx} {
  #          GiD_Process Mescape Results Graphs OptionsGraph CoordType Options PatternRadMarks [lindex $all_radpat $idx]
  #  }
  #}
    
  #set cur [lindex [lindex [ GiD_Info postprocess get graphs_option PatRad] 0] 0]
  #  if {$cur ne $data(opt,patradmarks)} {
  #      GiD_Process Mescape Results Graphs OptionsGraph CoordType Options PatternRadMarks $data(opt,patradmarks)
  #}

  # set idx [$data(wid,cbradpat) getvalue]
  set idx [$data(wid,cbradpat) current]
  if {$idx != -1} {
	set radpat [lindex [ GiD_Info postprocess get graphs_option PatRad] 0]
	set all_radpat [lindex $radpat 1]
	set cur_radpat_idx [lsearch $all_radpat [lindex $radpat 0]]
	if {$cur_radpat_idx != $idx} {
	    GiD_Process Mescape Results Graphs OptionsGraph CoordType Options PatternRadMarks \
		[lindex $all_radpat $idx]
    }
	}
    
  GiD_Process Mescape Results Graphs OptionsGraph CoordType $data(opt,coord)

  set data(tmp,applying) 0
  FillCOptions
  FillGOptions
  UpdateOrigColor
  UpdateRadColor
}

proc ::PostGraphs::ApplyLOptions { } {
  variable data

  set data(tmp,applying) 1
  set data(tmp,changename) ""
  set need_update 0
  set cur_sel [$data(wid,lblines) curselection]
  set len_sel [llength $cur_sel]


  foreach l_idx $cur_sel {
    set g [lindex $data(opt,lines,all) $l_idx]
    # Style
    # Color
    # CFill
    # LineWidth
    # Pattern
    # PatternFactor
    # PointSize
    # Title
    foreach {opt name} {stylegraphs style colorgraphs color \
		            colorascfill cfill \
		            linewidthgraphs lwidth \
		            patterngraphs pat patternfactor patfact \
		            pointsize psize titlegraphs lname} {
      if {$data(opt,chk$name,value)} {
	set need_update 1
	if {$name eq "pat"} {
	  set v [lindex $data(opt,pattern,all,names) \
		     [ $data(wid,cbpat) current ]]
	} elseif {$name eq "style"} {
	  set v [lindex $data(opt,style,all,names) \
		     [$data(wid,cbstyle) current ]]
	} else {
	  set v $data(opt,$name,value)
	}
	if {$name eq "lname"} {
	  set v [regsub " " $v "_"]
	  set data(tmp,changename) $v
	
	}

	GiD_Process escape escape escape result graphs optionsgraph $opt $g $v
		
	set hay_cfill [ GiD_Info postprocess get graphs_option HayCFill]
	set hay_cfill [lindex $hay_cfill 0]
	set prova [lindex $hay_cfill 0]
		
	if { ( $name eq "cfill") && ( $::PostGraphs::data(opt,cfill,value) eq "Yes") && ( $prova eq "No" )} {
	   tk_dialogRAM $data(wid,main).tempwin [_ "Error window"] \
	   [_ "Please try to load contour fill results first"] \
	    error 0 [_ "OK"]
	}

      }
    }
  }
  foreach name {style color cfill lwidth pat patfact psize lname} {
    set data(opt,chk$name,value) 0
  }
  set data(tmp,applying) 0
  if {$need_update} {
    FillLOptions
  }
  set data(tmp,changename) ""
}

proc ::PostGraphs::ApplyCreate { {node ""} } {
  variable data

  set view [GetCurrentView]
  if {$view eq "No_Graph"} {
    GiD_Process escape escape escape result graphs optionsgraph cleargraphs
  } else {
    if {[AllArgumentsOk]} {
      ChangeAnalysis
      GiD_Process Mescape {*}$data(cmds,args)
      FinishButton [winfo toplevel $data(wid,fcmd)] $data(wid,fcmd) \
	  [_ "Press 'Finish' to end selection"] \
	  ::PostGraphs::RestoreAnalysis "disableall" \
	  [ GiD_Set SmallWinSelecting]
    }
  }
}

proc ::PostGraphs::AllArgumentsOk { } {
  variable data

  set view [GetCurrentView]

  if { ![info exists data(cmds,$view)] } {
      return no
  }

  set data(cmds,args) [list result graphs $data(cmds,$view)]
    
  if { $view == "Integrate" } {
    lappend data(cmds,args) $::GidPriv(PostSpaceDimension) AllSteps ScalarResult
  }
    
  foreach a {X Y} {
    set tree $data(wid,tree,$a)
    set sel [$tree selection get]
    if {!$data(cmds,$view,$a,require)} {
      continue
    }
    if {![llength $sel]} {
      return no
    }
    set node [lindex $sel 0]
    set data_node [$tree itemcget $node -data]
    eval lappend data(cmds,args) $data_node
    if {$data(cmds,$view,$a,require)==2 && [llength $data_node]==1} {
      lappend data(cmds,args) $data_node
    }
  }

  if { $view == "Point_Graph" && $::ViewMenuPriv(pointgraphallsteps) } {
      lappend data(cmds,args) AllSteps
  }

  return yes
}

proc ::PostGraphs::ChangeAnalysis { } {
  variable data

  set data(analysis,save) [ GiD_Info postprocess get cur_analysis]
  set data(analysis,step,save) [ GiD_Info postprocess get cur_step]
  GiD_Process escape escape escape escape escape results analysissel \
      [GetCurrentAnalysis] [GetCurrentStep]
}

proc ::PostGraphs::RestoreAnalysis { } {
  variable data

  GiD_Process escape escape escape escape escape results analysissel \
      $data(analysis,save) $data(analysis,step,save)
}


proc ::PostGraphs::ChangeAutomaticRedraw {} {
  variable data

  set data(auto_redraw,save) [GiD_Set AutomaticRedraw]
  GiD_Process escape escape escape escape escape escape utilities variables \
      AutomaticRedraw -1 escape escape escape escape
}

proc ::PostGraphs::RestoreAutomaticRedraw {} {
  variable data

  GiD_Process escape escape escape escape escape escape utilities variables \
      AutomaticRedraw $data(auto_redraw,save) escape escape escape escape
}

proc ::PostGraphs::OnClose { } {
  variable data

  destroy $data(wid,main)
}

proc ::PostGraphs::CreateAxis { f } {
  variable data

  panedwindow $f.pn -orient horizontal
  LabelFrame $f.pn.x -side top -text [_ "X axis"]: -anchor w \
      -background $data(activebg)
  set data(wid,lf,X) $f.pn.x
  CreateResultTree [$f.pn.x getframe] X
  LabelFrame $f.pn.y -side top -text [_ "Y axis"]: -anchor w \
      -background $data(activebg)
  set data(wid,lf,Y) $f.pn.y
  CreateResultTree [$f.pn.y getframe] Y
  $f.pn add $f.pn.x $f.pn.y -sticky "snew"
  grid $f.pn -sticky "snew"
  grid rowconfigure $f 0 -weight 1
  grid columnconfigure $f 0 -weight 1
}

proc ::PostGraphs::UpdateResults { } {
  variable data

  switch [GetCurrentView] {
    Point_Evolution {
      set msg [_ "All step at %s" [ $data(wid,cbana) get ] ]
      ClearTree X $msg
      $data(wid,lf,X) configure -text [_ "X axis: All steps"]
      FillAxis Y
    }
    Point_Graph {
      FillAxis X
      FillAxis Y
    }
    Border_Graph {
      FillBorderGraph
    }
    Line_Graph {
      ClearTree X [_ "line length"]
      $data(wid,lf,X) configure -text [_ "X axis: line length"]
      FillAxis Y
    }
    Integrate {
      set msg [_ "All step at %s" [ $data(wid,cbana) get ] ]
      ClearTree X $msg
      $data(wid,lf,X) configure -text [_ "X axis: All steps"]
      FillAxis Y
    }
    default {
      FillResults $data(wid,tree,X) ""
      FillResults $data(wid,tree,Y) ""
    }
  }
}

proc ::PostGraphs::OnSelectNode { Axis node } {
  variable data

  set tree $data(wid,tree,$Axis)
  if {[llength [set info [$tree itemcget $node -data]]]}  {
    $tree selection set $node
    set txt [$tree itemcget $node -text]
    $data(wid,lf,$Axis) configure -text [_ "%s axis: %s" $Axis $txt]
  } else {
    if {[$tree itemcget $node -open]} {
      $tree closetree $node
    } else {
      $tree opentree $node
    }
  }
}

proc ::PostGraphs::CreateResultTree {f Axis}  {
  variable data

  ScrolledWindow $f.sw -bd 0 -auto both
  set t [Tree $f.sw.t]
  $t bindImage <Control-1> {}
  $t bindText <Control-1> {}
  $t bindImage <1> "::PostGraphs::OnSelectNode $Axis"
  $t bindText <1> " ::PostGraphs::OnSelectNode $Axis"
  $t bindImage <Double-Button-1> ::PostGraphs::ApplyCreate
  $t bindText <Double-Button-1> ::PostGraphs::ApplyCreate

  set data(wid,tree,$Axis) $t
  $f.sw setwidget $t
  pack $f.sw -fill both -expand yes
}

proc ::PostGraphs::ClearTree { axis msg } {
  variable data

  set tree $data(wid,tree,$axis)
  $tree delete [$tree nodes root]
  if {$msg ne ""} {
    $tree insert end root "" -text $msg
  }
}

proc ::PostGraphs::FillResults {tree results} {
  variable data

  $tree delete [$tree nodes root]
  set view [GetCurrentView]
  set analysis [GetCurrentAnalysis]
  set step [GetCurrentStep]

  $tree delete [$tree nodes root]
  if {![llength $results]} {
    set txt [_ "<No usable results found>"]
    $tree insert end root "" -text $txt
    return
  }

  foreach r $results {
    regsub -all // $r " " levels
    set nodep root
    foreach l [lrange $levels 0 end-1] {
      set nodeid "$nodep//[PRToNode $l]"
      if {![$tree exists $nodeid]} {
	set txt [= [regsub -all _ $l " "]]
	$tree insert end $nodep $nodeid -text $txt -image [GetImage folder.gif]
      }
      set nodep $nodeid
    }
    set components [ GiD_Info postprocess get components_list $view $r $analysis $step]
    set txt [= [regsub -all _ [lindex $levels end] " "]]
    if {[llength $components] > 1} {
      $tree insert end $nodep [PRToNode $r] -text $txt -image [GetImage prof1.gif]
      set nodep [PRToNode $r]
      foreach c $components {
	set txt [= [regsub -all _ $c " "]]
	$tree insert end $nodep "$nodep//[PRToNode $c]" \
	    -text $txt -image [GetImage point16x16.gif] \
	    -data [list $r $c]
      }
    } else {
      $tree insert end $nodep [PRToNode $r] -text $txt \
	  -image [GetImage point16x16.gif] -data [list $r]
    }
  }
}

proc ::PostGraphs::FillAxis { axis } {
  variable data

  set tree $data(wid,tree,$axis)
  set view [GetCurrentView]
  set analysis [GetCurrentAnalysis]
  set step [GetCurrentStep]

  set results {}
  catch {
    set results [ GiD_Info postprocess get results_list $view $analysis $step]
  }
  $data(wid,lf,$axis) configure -text [_ "%s axis" $axis]:
  FillResults $tree $results
}

proc ::PostGraphs::OnChangeView { {force 0} } {
  variable data

  set cur_view [GetCurrentView]
  if {$force || ![info exists data(view,last)] || $data(view,last) ne $cur_view} {
    set data(view,last) $cur_view
    UpdateResults
  }
  if { $cur_view == "Point_Graph" } {
      grid $data(wid,chkallsteps)
  } else {
      grid remove $data(wid,chkallsteps)
  }
}

proc ::PostGraphs::FillBorderGraph { } {
  variable data

  $data(wid,lf,X) configure -text [_ "X axis"]:
  set tree $data(wid,tree,X)
  $tree delete [$tree nodes root]
  lappend options X_Variation [_ "X Variation"]
  lappend options Y_Variation [_ "Y Variation"]
  lappend options Z_Variation [_ "Z Variation"]
  lappend options LineVariation [_ "Line Variation"]
  foreach {x lx} $options {
    $tree insert end root $x -text $lx -image [GetImage point16x16.gif] -data [list $x]
  }
  FillAxis Y
}

proc ::PostGraphs::OnChangeAnalysis { } {
  variable data

  set cur_ana [GetCurrentAnalysis]
  if {![info exists data(analysis,last)] || \
	  $data(analysis,last) ne $cur_ana} {
    set data(analysis,last) $cur_ana
    UpdateResults
  }
}

proc ::PostGraphs::OnChangeStep { } {
  variable data

  if {![info exists data(step,last)] || $data(step,last) ne $data(step)} {
    set data(step,last) $data(step)
    UpdateResults
  }
}

proc ::PostGraphs::CreateView { f } {
  variable data

  label $f.lview -text [_ "View"]:
  #ComboBox $f.cbview -textvariable ::PostGraphs::data(view) -editable 0 -bd 1 \
  #    -modifycmd ::PostGraphs::OnChangeView
  TTKComboBox $f.cbview -textvariable ::PostGraphs::data(view) \
      -state readonly
  set data(wid,cbview) $f.cbview
  # no entiendo por que TTKComboBox lo invoca cuando se cambia la propiedad.
  after idle "$f.cbview configure -modifycmd ::PostGraphs::OnChangeView"

  label $f.lana -text [_ "Analysis"]:
  #ComboBox $f.cbana -textvariable ::PostGraphs::data(analysis) -editable 0  \
  #    -bd 1 -modifycmd ::PostGraphs::OnChangeAnalysis
  TTKComboBox $f.cbana -textvariable ::PostGraphs::data(analysis) \
      -state readonly -modifycmd ::PostGraphs::OnChangeAnalysis
  set data(wid,cbana) $f.cbana

  label $f.lstep -text [_ "Step"]:
  TTKComboBox $f.cbstep -textvariable ::PostGraphs::data(step) \
      -state readonly -modifycmd ::PostGraphs::OnChangeStep
  #ComboBox $f.cbstep -textvariable ::PostGraphs::data(step) -editable 0 -bd 1 \
  #    -modifycmd ::PostGraphs::OnChangeStep
  set data(wid,cbstep) $f.cbstep


  checkbutton $f.chkallsteps -text [_ "All steps"] -variable ::ViewMenuPriv(pointgraphallsteps)
  set data(wid,chkallsteps) $f.chkallsteps
  if { ![info exists ::ViewMenuPriv(pointgraphallsteps)] } {
      set ::ViewMenuPriv(pointgraphallsteps) 1
  }

  FillViewFrame

  grid $f.lview $f.cbview $f.lana $f.cbana $f.lstep $f.cbstep $f.chkallsteps -sticky ew
  grid columnconfigure $f {1 3 5} -weight 1
  grid remove $f.chkallsteps
}

proc ::PostGraphs::FillViewFrame { } {
  variable data

  if {[Alive]} {
    FillAnalysis
    FillSteps
    FillViews
  }
}

proc ::PostGraphs::FillAnalysis { } {
  variable data

  set all [lsort -dictionary [ GiD_Info postprocess get all_analisis]]
  # set all [ GiD_Info postprocess get all_analisis]
  set curr [ GiD_Info postprocess get cur_analisis]
  set data(analysis,all) $all

  if {![llength $all]} {
    ClearView
  } else {
    # translate the analysis
    set i 0
    set icur 0
    foreach a $all {
      if {$a eq $curr} {
	set icur $i
      }
      lappend all_trans [regsub -all _ [= $a] " "]
      incr i
    }
    $data(wid,cbana) configure -values $all -labels $all_trans
    #TTKMenubuttonConfigure $data(wid,cbana) -values $all_trans
    #$data(wid,cbana) setvalue "@$icur"
    $data(wid,cbana) set $curr
  }
}

proc ::PostGraphs::ClearTTKCombo { cb } {
  $cb set ""
  $cb configure -values "" -labels ""
}

proc ::PostGraphs::ClearView { } {
  variable data
  ClearTTKCombo $data(wid,cbana)
  ClearTTKCombo $data(wid,cbstep)
  ClearTTKCombo $data(wid,cbview)
  array set data {
    view ""
    view,last ""
    analysis ""
    analysis,last ""
    step ""
    step,last ""
  }
}

proc ::PostGraphs::GetCurrentAnalysis { } {
  variable data
  #lindex $data(analysis,all) [$data(wid,cbana) getvalue]
  #$data(wid,cbana) get
  set data(analysis)
}

proc ::PostGraphs::GetCurrentView { } {
  variable data
  #lindex $data(view,all) [$data(wid,cbview) getvalue]
  #$data(wid,cbview) get ;#this return the translated string, not the keyword !!
   set data(view)
}

proc ::::PostGraphs::GetCurrentStep { } {
  variable data
  set data(step)
}

proc ::PostGraphs::FillSteps { } {
  variable data

  set cur_ana [GetCurrentAnalysis]
  if {$cur_ana ne ""} {
    set curr_st [ GiD_Info postprocess get cur_step $cur_ana]
    set all_st [ GiD_Info postprocess get all_steps $cur_ana]
    $data(wid,cbstep) configure -values $all_st
    #$data(wid,cbstep) setvalue "@[lsearch $all_st $curr_st]"
    $data(wid,cbstep) set $curr_st
  } else {
    ClearView
  }
}

proc ::PostGraphs::FillViews { } {
  variable data

  set cur_ana [GetCurrentAnalysis]
  if {$cur_ana ne ""} {
    set all [ GiD_Info postprocess get all_graphs_views]
    set data(view,all) $all
    #bloque de traduccion: solo para que RamTranslator los encuentre#
      list [_ "No Graph"] [_ "Point Evolution"] [_ "Point Graph"] \
	[_ "Border Graph"] [_ "Line Graph"] [_ "Integrate"]
    #fin bloque de traduccion#
    foreach v $all {
      lappend all_trans [_ [regsub -all _ $v " "]]
    }
    $data(wid,cbview) configure -values $all -labels $all_trans
    # TTKMenubuttonConfigure $data(wid,cbview) -values $all_trans
    if {[info exists data(view,last)] && $data(view,last) ne ""} {
      #$data(wid,cbview) setvalue @[lsearch $all $data(view,last)]
      $data(wid,cbview) set $data(view,last)
    } else {
      set data(view,last) ""
      #$data(wid,cbview) setvalue first
      $data(wid,cbview) set [ lindex $all 0 ]
    }
  } else {
    set data(view,all) ""
    ClearView
  }
}

proc ::PostGraphs::RefreshFromGiD { } {
  variable data

  if {$data(tmp,applying)} {
  } else {
    after idle {
      namespace eval ::PostGraphs {
	if { [ Alive]} {
	  FillGOptions
	  FillCOptions
	  FillLOptions
	}
      }
    }
  }
}

proc ::PostGraphs::ToggleView { } {
  ::TogglePostGraph
}

# TODO:
#
#
