
proc getColorMask { str var} {
    set intMask [ scan $str "%d"]
    set ${var}(red) 0
    set ${var}(green) 0
    set ${var}(blue) 0
    set ${var}(alpha) 0
    if { $intMask >= 1000} {
	set ${var}(red) 1
	incr intMask -1000
    }
    if { $intMask >= 100} {
	set ${var}(green) 1
	incr intMask -100
    }
    if { $intMask >= 10} {
	set ${var}(blue) 1
	incr intMask -10
    }
    if { $intMask >= 1} {
	set ${var}(alpha) 1
	incr intMask -1
    }
}

proc strColorMask { var} {
    set msk ""
    foreach idx {red green blue alpha} {
	if { [ subst $${var}($idx)]} {
	    append msk "1"
	} else {
	    append msk "0"
	}
    }
    return $msk
}

proc StereoGetInfo { } {
    array set ::GID_Stereo [ GiD_Info stereo]
    # copy variables to check user changes
    foreach idx [ list Stereo StereoMode EyeDistance FlipEyes \
		      RelativeEyeDistance LeftEyeColorMask RightEyeColorMask] {
	set ::GID_Stereo(Old$idx) $::GID_Stereo($idx)
    }
    getColorMask $::GID_Stereo(LeftEyeColorMask) ::GID_Stereo_Left_Mask
    getColorMask $::GID_Stereo(RightEyeColorMask) ::GID_Stereo_Right_Mask
    if { ![ info exists ::GID_Stereo(DynamicApply)]} {
	set ::GID_Stereo(DynamicApply) 0
    }
    
    # NORMAL o PERSPECTIVE
    set ::GID_Stereo(ModoPerspectiva) [ GiD_Info perspectivemode ]
    set ::GID_Stereo(OldModoPerspectiva) $::GID_Stereo(ModoPerspectiva)

    # Shadow options
    array set ::GID_Shadow [ GiD_Info shadow]
    # copy variables to check user changes
    foreach idx [ list Shadow Render ShadowBias ShowDepthMap ShadowMode UseFBO FBOSize DimmedLightFactor] {
	set ::GID_Shadow(Old$idx) $::GID_Shadow($idx)
    }
    # set idx [ lsearch $::GID_Shadow(FBOSizeOptions) $::GID_Shadow(FBOSize)]
    # set ::GID_Shadow(FBOSizeIdx) $idx
    # set ::GID_Shadow(OldFBOSizeIdx) $::GID_Shadow(FBOSizeIdx)
}

proc CreateLabelMenuButtonFrame { w txt lst_lbl lst_val var} {
    frame $w -bd 0
    label $w.lbl -text $txt
    TTKMenubutton $w.mb -labels $lst_lbl -values $lst_val -textvariable $var
    grid $w.lbl $w.mb -sticky nes
    grid configure  $w.mb -sticky nw
    grid rowconfigure $w 0 -weight 1
    grid columnconfigure $w 0 -weight 1
}

proc CreateLabelMenuButton { w txt lst_lbl lst_val var w_top} {
    label ${w}_lbl -text $txt
    TTKMenubutton ${w}_mb -labels $lst_lbl -values $lst_val -textvariable $var \
	-modifycmd "StereoDynamicApply $w_top ;\#"
    # grid ${w}_lbl ${w}_mb -sticky nes
    # grid configure  ${w}_mb -sticky nw
    # grid rowconfigure $w 0 -weight 1
    # grid columnconfigure $w 0 -weight 1
}

proc CreateColorMaskCB { w var} {
    frame $w
    foreach idx {red green blue alpha} lbl [list [_ "red"] [_ "green"] [_ "blue"] [_ "alpha"]] {
	ttk::checkbutton ${w}.$idx -text $lbl -variable ${var}($idx) \
	-command "StereoDynamicApply $w ;\#"
    }
    grid ${w}.red ${w}.green ${w}.blue ${w}.alpha -padx 4
}

proc CheckSoftwareOpenGL { w txt} {
    if { $::GidPriv(SoftwareOpenGL)} {
	set msg $txt
	append msg " " [_ "Open preferences window to change OpenGL mode?"]
	set resp [ tk_messageBox -parent $w -default yes -title [_ "Change OpenGL mode"] \
		       -type yesno -message $msg]
	if { $resp == "yes"} {
	    ChangeVariables .gid.wvar Graphical
	}
    }
}

proc StereoApply { w } {
    ##############################
    # Stereo options
    ############################## 

    # para salir de todos los menus, que a veces no entiende 'View'
    GiD_Process escape escape escape escape escape escape

    # First the options
    if { $::GID_Stereo(OldModoPerspectiva) != $::GID_Stereo(ModoPerspectiva)} {
	GiD_Process Projection $::GID_Stereo(ModoPerspectiva)
	set ::GID_Stereo(OldModoPerspectiva) $::GID_Stereo(ModoPerspectiva)
    }
    set ::GID_Stereo(LeftEyeColorMask) [ strColorMask ::GID_Stereo_Left_Mask]
    set ::GID_Stereo(RightEyeColorMask) [ strColorMask ::GID_Stereo_Right_Mask]
    foreach idx [ list EyeDistance FlipEyes \
		      RelativeEyeDistance LeftEyeColorMask RightEyeColorMask] {
	if { $::GID_Stereo(Old$idx) != $::GID_Stereo($idx)} {
	    GiD_Process View Stereo $idx $::GID_Stereo($idx) escape escape
	    # WarnWinText "GiD_Process View Stereo $idx $::GID_Stereo($idx) escape escape"
	    set ::GID_Stereo(Old$idx) $::GID_Stereo($idx)
	}
    }
    if { $::GID_Stereo(OldStereoMode) != $::GID_Stereo(StereoMode)} {
	if { "$::GID_Stereo(StereoMode)" == "QuadBuffers"} {
	    set qb_trad [_ "Quad buffers"]
	    CheckSoftwareOpenGL $w [_ "%s does not work with Software OpenGL." $qb_trad]
	}
	GiD_Process View Stereo $::GID_Stereo(StereoMode) escape escape
	# WarnWinText "GiD_Process View Stereo $val escape escape"
	set ::GID_Stereo(OldStereoMode) $::GID_Stereo(StereoMode)
    }

    # then shitch mode on or off
    if { $::GID_Stereo(OldStereo) != $::GID_Stereo(Stereo)} {
	set val "No"
	if { $::GID_Stereo(Stereo)} {
	    set val "Yes"
	}
	GiD_Process View Stereo $val escape escape
	# WarnWinText "GiD_Process View Stereo $val escape escape"
	set ::GID_Stereo(OldStereo) $::GID_Stereo(Stereo)
    }
    GiD_Process escape escape escape escape escape escape

    ##############################
    # Stereo options
    ############################## 

    # First the options
    foreach idx [ list Render ShowDepthMap ShadowMode UseFBO] {
	if { $::GID_Shadow(Old$idx) != $::GID_Shadow($idx)} {
	    # sustituimos 0,1 por si valor
	    set val [ lindex $::GID_Shadow(${idx}Options) $::GID_Shadow($idx)]
	    GiD_Process View Shadow $val escape escape
	    set ::GID_Shadow(Old$idx) $::GID_Shadow($idx)
	}
    }
    if { $::GID_Shadow(OldShadowBias) != $::GID_Shadow(ShadowBias)} {
	GiD_Process View Shadow ShadowBias $::GID_Shadow(ShadowBias) escape escape
	set ::GID_Shadow(OldShadowBias) $::GID_Shadow(ShadowBias)
    }
    if { $::GID_Shadow(OldDimmedLightFactor) != $::GID_Shadow(DimmedLightFactor)} {
	GiD_Process View Shadow DimmedLightFactor $::GID_Shadow(DimmedLightFactor) escape escape
	set ::GID_Shadow(OldDimmedLightFactor) $::GID_Shadow(DimmedLightFactor)
    }

    # if { $::GID_Shadow(OldFBOSizeIdx) != $::GID_Shadow(FBOSizeIdx)} {
    # 	set ::GID_Shadow(FBOSize) [ lindex $::GID_Shadow(FBOSizeOptions) $::GID_Shadow(FBOSizeIdx)]
    # 	GiD_Process View Shadow FBOSize $::GID_Shadow(FBOSize) escape escape
    # 	set ::GID_Shadow(OldFBOSizeIdx) $::GID_Shadow(FBOSizeIdx)
    # }

    if { $::GID_Shadow(OldFBOSize) != $::GID_Shadow(FBOSize)} {
	GiD_Process View Shadow FBOSize $::GID_Shadow(FBOSize) escape escape
	set ::GID_Shadow(OldFBOSize) $::GID_Shadow(FBOSize)
    }

    # then shitch mode on or off
    if { $::GID_Shadow(OldShadow) != $::GID_Shadow(Shadow)} {
	set val "Off"
	if { $::GID_Shadow(Shadow)} {
	    set val "On"
	}
	GiD_Process View Shadow $val escape escape
	set ::GID_Shadow(OldShadow) $::GID_Shadow(Shadow)
	if { $::GID_Shadow(Shadow)} {
	    StereoGetInfo
	}
    }

    GiD_Process escape escape escape escape escape escape
}

proc StereoDynamicApply { w} {
    if { $::GID_Stereo(DynamicApply)} {
	StereoApply $w
    }
}

proc StereoClose { w } {
    destroy $w
}

proc StereoW { { w .gid.wStereoW} } {
    
    InitWindow $w [_ "Advanced viewing settings"] PrePostStereoWindowGeom StereoW
    if { ![ winfo exists $w] } return ;# windows disabled || usemorewindows == 0
    #toplevel $w

    StereoGetInfo

    set top $w.top
    set but $w.but
    CreateApplyCloseDecorationWindow $w $top $but "StereoApply $w" "StereoClose $w"
    # CreateDecorationWindow $w $top $but \
    # 	[ list [_ "Apply"] [_ "Close"]] \
    # 	[ list "StereoApply $w" "StereoClose $w"] \
    # 	[ list <Return> <Escape>]

    ########################################
    # Stereo Frame
    ########################################

    set topStereo [ frame $top.fStereo -relief ridge -bd 2]

    label $topStereo.lbl -text [_ "Stereo settings"]:

    ttk::checkbutton $topStereo.useStereo -variable ::GID_Stereo(Stereo) -text [_ "Use stereo"] \
	-command "StereoDynamicApply $w ;\#"

    frame $topStereo.f
    ttk::checkbutton $topStereo.f.perspCB -text [_ "Perspective"] \
	-variable ::GID_Stereo(ModoPerspectiva) \
	-onvalue PERSPECTIVE -offvalue NORMAL \
	-command "StereoDynamicApply $w ;\#"
    ttk::button $topStereo.f.perspOpt -text "-->" -command PerspectiveW -width 5

    grid $topStereo.f.perspCB $topStereo.f.perspOpt -sticky en

    CreateLabelMenuButton $topStereo.stereo_mode [_ "stereo mode"]: \
	[ list [_ "Anaglyph colors"] [_ "Quad buffers"]] \
	[ list AnaglyphColors QuadBuffers] \
	::GID_Stereo(StereoMode) $w

    #scale ....
    label $topStereo.eye_dist_lbl -text [_ "eye distance"]:
    scale $topStereo.eye_dist_sc -from 0.0 -to 0.1 -resolution 0.005 \
	-variable ::GID_Stereo(EyeDistance) -orient horizontal \
	-command "StereoDynamicApply $w ;#"

    ttk::checkbutton $topStereo.flipEyes -variable ::GID_Stereo(FlipEyes) -text [_ "switch eyes"] \
	-command "StereoDynamicApply $w ;\#"

    CreateLabelMenuButton $topStereo.relative_distance [_ "eye distance mode"]: \
	[ list [_ "scaled with the model"] [_ "absolute eye distance"]] \
	[ list 1 0] \
	::GID_Stereo(RelativeEyeDistance) $w

    label $topStereo.left_mask_lbl -text [_ "left color mask"]:
    CreateColorMaskCB $topStereo.left_mask_cb ::GID_Stereo_Left_Mask

    label $topStereo.right_mask_lbl -text [_ "right color mask"]:
    CreateColorMaskCB $topStereo.right_mask_cb ::GID_Stereo_Right_Mask

    # empaquetamiento # -columnspan 2
    grid $topStereo.useStereo $topStereo.f -sticky w -padx 8 -pady 4
    grid configure $topStereo.f -sticky e 
    # grid $topStereo.lbl -sticky w -columnspan 2
    #grid $topStereo.stereo_mode -sticky w -padx 8
    grid $topStereo.stereo_mode_lbl $topStereo.stereo_mode_mb -sticky nes
    grid configure $topStereo.stereo_mode_mb -sticky nw

    grid $topStereo.eye_dist_lbl $topStereo.eye_dist_sc -sticky nes
    grid configure $topStereo.eye_dist_sc -sticky nwe

    grid $topStereo.flipEyes -sticky w -row 3 -column 1
    # grid $topStereo.relative_distance -sticky w -padx 8
    grid $topStereo.relative_distance_lbl $topStereo.relative_distance_mb -sticky nes
    grid configure $topStereo.relative_distance_mb -sticky nw

    grid $topStereo.left_mask_lbl $topStereo.left_mask_cb -sticky nes
    grid configure $topStereo.left_mask_cb -sticky nw

    grid $topStereo.right_mask_lbl $topStereo.right_mask_cb -sticky nes
    grid configure $topStereo.right_mask_cb -sticky nw


    ########################################
    # Shadow Frame
    ########################################

    set topShadow [ frame $top.fShadow -relief ridge -bd 2]

    label $topShadow.lbl -text [_ "Shadow settings"]:

    # Shadow on / off
    ttk::checkbutton $topShadow.useShadow -variable ::GID_Shadow(Shadow) \
	-text [_ "Use shadow"] \
	-command "StereoDynamicApply $w ;\#"

    label $topShadow.shadow_mode_lbl -text [_ "shadow color"]:
    ttk::frame $topShadow.shadow_mode_f
    ttk::radiobutton $topShadow.shadow_mode_f.rd1 \
	-variable ::GID_Shadow(ShadowMode) \
	-text [_ "black#C#shadow color"] \
	-value 0 \
	-command "StereoDynamicApply $w ;\#"
    ttk::radiobutton $topShadow.shadow_mode_f.rd2 \
	-variable ::GID_Shadow(ShadowMode) \
	-text [_ "ambient#C#shadow color"] \
	-value 1 \
	-command "StereoDynamicApply $w ;\#"
    scale $topShadow.shadow_mode_f.dimmed_light_sc \
	-from [ lindex $::GID_Shadow(DimmedLightFactorOptions) 0] \
	-to [ lindex $::GID_Shadow(DimmedLightFactorOptions) 1] \
	-resolution 0.005 \
	-variable ::GID_Shadow(DimmedLightFactor) \
	-orient horizontal -sliderlength 8 \
	-command "StereoDynamicApply $w ;#"
    grid $topShadow.shadow_mode_f.rd1 -row 0 -column 0 -sticky nw
    grid $topShadow.shadow_mode_f.dimmed_light_sc -row 0 -column 1 -rowspan 2 -sticky nwe
    grid $topShadow.shadow_mode_f.rd2 -row 1 -column 0 -sticky nw
    grid rowconfigure $topShadow.shadow_mode_f 0 -weight 1
    grid columnconfigure $topShadow.shadow_mode_f 1 -weight 1

    #scale .... shadow bias
    label $topShadow.depth_bias_lbl -text [_ "shadow bias"]:
    scale $topShadow.depth_bias_sc \
	-from [ lindex $::GID_Shadow(ShadowBiasOptions) 0] \
	-to [ lindex $::GID_Shadow(ShadowBiasOptions) 1] \
	-resolution 0.001 \
	-variable ::GID_Shadow(ShadowBias) -orient horizontal \
	-command "StereoDynamicApply $w ;#"

    ttk::frame $topShadow.fbo_opt
    ttk::checkbutton $topShadow.fbo_opt.useFBO -variable ::GID_Shadow(UseFBO) \
	-text [_ "finer silhouette (FramebufferObject)"] \
	-command "StereoDynamicApply $w ;\#"
    # scale $topShadow.fbo_opt.fbo_size_sc \
    # 	-from 0 \
    # 	-to [ expr [ llength $::GID_Shadow(FBOSizeOptions)] - 1] \
    # 	-resolution 1 \
    # 	-variable ::GID_Shadow(FBOSizeIdx) -orient horizontal \
    # 	-command "StereoDynamicApply $w ;#"
    ttk::label $topShadow.fbo_opt.fbo_size_lbl -text [_ "quality"]:
    TTKComboBox $topShadow.fbo_opt.fbo_size_sc \
	-labels [ list [_ "medium#C#Quality"] [_ "high#C#Quality"] [_ "very high#C#Quality"] [_ "highest#C#Quality"]] \
	-values [ list 1024 2048 4096 8192] \
	-textvariable ::GID_Shadow(FBOSize) \
	-state readonly \
	-modifycmd "StereoDynamicApply $w ;#"
     
    grid $topShadow.fbo_opt.useFBO $topShadow.fbo_opt.fbo_size_lbl $topShadow.fbo_opt.fbo_size_sc -sticky w -padx 0 -pady 0
    grid configure $topShadow.fbo_opt.fbo_size_lbl -sticky e -padx 4 -pady 0
    grid configure $topShadow.fbo_opt.fbo_size_sc -sticky ew -padx 4 -pady 0
    grid rowconfigure $topShadow.fbo_opt 0 -weight 1
    grid columnconfigure $topShadow.fbo_opt 2 -weight 1

    grid $topShadow.useShadow -sticky w -padx 8 -pady 4 -columnspan 2
    grid $topShadow.shadow_mode_lbl $topShadow.shadow_mode_f -sticky nes
    grid configure $topShadow.shadow_mode_f -sticky new
    grid $topShadow.depth_bias_lbl $topShadow.depth_bias_sc -sticky nes
    grid configure $topShadow.depth_bias_sc -sticky new
    grid $topShadow.fbo_opt -sticky new -padx 8 -pady 4 -columnspan 2

    grid columnconfigure $topShadow 1 -weight 1
    # grid columnconfigure $topShadow 1 -weight 1

    ########################################

    ttk::checkbutton $top.dynamicUpdate -variable ::GID_Stereo(DynamicApply) \
	-text [_ "Dynamic update"]

    grid $topStereo -sticky news
    grid $topShadow -sticky news
    grid $top.dynamicUpdate -sticky w  -padx 8 -pady 8
    grid columnconfigure $top 0 -weight 1
    grid rowconfigure $top 0 -weight 1
    grid rowconfigure $top 1 -weight 1
    grid rowconfigure $top 2 -weight 1
    grid rowconfigure $top 3 -weight 1

}